unit DirectoryMenu;

{
  EasyMenus
  Copyright 2005 Murariu Alexandru
  muralexandru@yahoo.com
  All rights reserved.
  For conditions of distribution and use, see LICENSE.TXT.

}

interface

uses
  Windows, Messages, SysUtils, Classes, Menus, Controls,
  SystemImageList, FileCtrl, Forms;

type

  TOnFileMenuItemClick = procedure (Sender : TObject; FileName : String) of Object;
  TOnIncludeItem = procedure (Sender : TObject; FileName : string; ISDir : boolean; var Include : boolean) of Object;

  TSpecialFolder = (spNone = 0, spDesktop = 1, spCommonDesktop = 2, spDesktopAll = 3, spFonts = 4, spMyDocs = 5, spCommonDocs = 6, spDocsAll = 7,
                   spPrograms = 8, spCommonPrograms = 9, spProgramsAll = 10, spStartMenu = 11, spCommonStartMenu = 12, spStartMenuAll = 13,
                   spStartUp = 14, spCommonStartUp = 15, spStartUpAll = 16, spSendTo = 17, spRecent = 18, spFavorites = 19, spCommonFavorites = 20, spFavoritesAll = 21,
                   spTemplates = 22, spCommonTemplates = 23, spAppData = 24, spCommonAppData = 25, spNetHood = 26);

  TDirectoryMenu = class(TComponent)
  private
    FromSP : Boolean;
    SP : TSpecialFolder;
    FDirMenuItem : TMenuItem;
    FImageList: TSystemImageList;
    FDir : string;
    FOnFileMenuItemClick : TOnFileMenuItemClick;
    FShowExt : boolean;
    FFileType : TFileType;
    FShowIcons : boolean;
    FOnIncludeItem : TOnIncludeItem;
    ItemMenuTag : Integer;
    FOnRefreshed: TNotifyEvent;
    FFilter: string;
    procedure QuickSort(AMenuItem : TMenuItem);
    procedure SETFDirMenuItem(Value : TMenuItem);
    procedure SETFDir(Value : string);
    procedure mClick(Sender: TObject);
    procedure SETFShowExt(Value : boolean);
    procedure SETFFileType(Value : TFileType);
    procedure SETFShowIcons(Value : boolean);
    procedure AddMenus(Item : TMenuItem; Dir : string);
    function GN(FileName : string) : string;
    procedure SETSP(Value : TSpecialFolder);
    function GetSpecialFolder(Dir : string) : TSpecialFolder;
    function GetFolderStrFromID(ID : TSpecialFolder) : string;
    function CanStorePath : boolean;
    procedure RemoveItems;
    procedure CreateItems;
    { Private declarations }
  protected
    procedure Notification(AComponent: TComponent;
    Operation: TOperation); override;
    procedure Loaded; override;
    { Protected declarations }
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Refresh;
    { Public declarations }
  published
    property ShowIcons : boolean read FShowIcons write SETFShowIcons;
    property FileType : TFileType read FFileType write SETFFileType;
    property Filter : string read FFilter write FFilter;
    property ShowFileExtension : boolean read FShowExt write SETFShowExt;
    property DirMenuItem : TMenuitem read FDirMenuItem write SETFDirMenuItem;
    property Directory : string read FDir write SETFDir stored CanStorePath;
    property OnFileMenuItemClick : TOnFileMenuItemClick read FOnFileMenuItemClick write FOnFileMenuItemClick;
    property OnIncludeItem : TOnIncludeItem read FOnIncludeItem write FOnIncludeItem;
    property SpecialFolder : TSpecialFolder read SP write SETSP;
    property OnRefreshed : TNotifyEvent read FOnRefreshed write FOnRefreshed;
    { Published declarations }
  end;

  resourcestring
  SEMPTY = '[Empty]';

  function SpecialFolderToString(SpecialFolder : TSpecialFolder) : string;

implementation
uses ShlObj, EM_Common;

function TDirectoryMenu.CanStorePath : boolean;
begin
result := GetSpecialFolder(FDir) = spNone;
end;

function SpecialFolderToString(SpecialFolder : TSpecialFolder) : string;
const
    CSIDL_COMMON_APPDATA = $0023;
    CSIDL_COMMON_TEMPLATES = $002D;
    CSIDL_COMMON_DOCUMENTS = $002E;
var S : string;
begin
 case SpecialFolder of
 spNone : result := '';
 spDesktop : result := GetShellFolderPath(CSIDL_DESKTOPDIRECTORY);
 spCommonDesktop : result := GetCommonShellFolder(CSIDL_COMMON_DESKTOPDIRECTORY, CSIDL_DESKTOPDIRECTORY);
 spDesktopAll :
            begin
            s := GetCommonShellFolder(CSIDL_COMMON_DESKTOPDIRECTORY, CSIDL_DESKTOPDIRECTORY);
            result := GetShellFolderPath(CSIDL_DESKTOPDIRECTORY);
            if (s <> '') and (s <> result) then result := result + ';' + s;
            end;
 spFonts : result := GetShellFolderPath(CSIDL_FONTS);
 spMyDocs : result := GetShellFolderPath(CSIDL_PERSONAL);
 spCommonDocs : result := GetCommonShellFolder(CSIDL_COMMON_DOCUMENTS ,CSIDL_PERSONAL);
 spDocsAll :
            begin
            s := GetCommonShellFolder(CSIDL_COMMON_DOCUMENTS ,CSIDL_PERSONAL);
            result := GetShellFolderPath(CSIDL_PERSONAL);
            if (s <> '') and (s <> result) then result := result + ';' + s;
            end;
 spPrograms : result := GetShellFolderPath(CSIDL_PROGRAMS);
 spCommonPrograms : result := GetCommonShellFolder(CSIDL_COMMON_PROGRAMS, CSIDL_PROGRAMS);
 spProgramsAll :
            begin
            s := GetCommonShellFolder(CSIDL_COMMON_PROGRAMS, CSIDL_PROGRAMS);
            result := GetShellFolderPath(CSIDL_PROGRAMS);
            if (s <> '') and (s <> result) then result := result + ';' + s;
            end;
 spStartMenu : result := GetShellFolderPath(CSIDL_STARTMENU);
 spCommonStartMenu : result := GetCommonShellFolder(CSIDL_COMMON_STARTMENU, CSIDL_STARTMENU);
 spStartMenuAll :
            begin
            s := GetCommonShellFolder(CSIDL_COMMON_STARTMENU, CSIDL_STARTMENU);
            result := GetShellFolderPath(CSIDL_STARTMENU);
            if (s <> '') and (s <> result) then result := result + ';' + s;
            end;
 spStartUp : result := GetShellFolderPath(CSIDL_STARTUP);
 spCommonStartUp : result := GetCommonShellFolder(CSIDL_COMMON_STARTUP, CSIDL_STARTUP);
 spStartUpAll :
            begin
            s := GetCommonShellFolder(CSIDL_COMMON_STARTUP, CSIDL_STARTUP);
            result := GetShellFolderPath(CSIDL_STARTUP);
            if (s <> '') and (s <> result) then result := result + ';' + s;
            end;
 spSendTo : result := GetShellFolderPath(CSIDL_SENDTO);
 spRecent : result := GetShellFolderPath(CSIDL_RECENT);
 spFavorites : result := GetShellFolderPath(CSIDL_FAVORITES);
 spCommonFavorites : result := GetCommonShellFolder(CSIDL_COMMON_FAVORITES, CSIDL_FAVORITES);
 spFavoritesAll :
            begin
            s := GetCommonShellFolder(CSIDL_COMMON_FAVORITES, CSIDL_FAVORITES);
            result := GetShellFolderPath(CSIDL_FAVORITES);
            if (s <> '') and (s <> result) then result := result + ';' + s;
            end;
 spTemplates : result := GetShellFolderPath(CSIDL_TEMPLATES);
 spCommonTemplates : result := GetCommonShellFolder(CSIDL_COMMON_TEMPLATES, CSIDL_TEMPLATES);
 spAppData : result := GetShellFolderPath(CSIDL_APPDATA);
 spCommonAppData : result := GetCommonShellFolder(CSIDL_COMMON_APPDATA, CSIDL_APPDATA);
 spNetHood : result := GetShellFolderPath(CSIDL_NETHOOD);
 end;
end;

function TDirectoryMenu.GetFolderStrFromID(ID: TSpecialFolder): string;
begin
  result := SpecialFolderToString(ID);
end;

procedure TDirectoryMenu.SETSP(Value : TSpecialFolder);
var S : string;
begin
SP := Value;
if Value = spNone then Exit;
FromSP := True;
try
S := GetFolderStrFromID(Value);
if S <> '' then SETFDir(S);
finally
FromSP := False;
end;
end;

function TDirectoryMenu.GetSpecialFolder(Dir : string) : TSpecialFolder;
var I : Integer;
    S : String;
begin
result := spNone;
for I := 1 to 26 do
    begin
    S := GetFolderStrFromID(TSpecialFolder(I));
    if S <> '' then if SameText(Dir,S) then
                                       begin
                                       result := TSpecialFolder(I);
                                       Exit;
                                       end;
    end;
end;

procedure TDirectoryMenu.SETFShowIcons(Value : boolean);
begin
FShowIcons := Value;
Refresh;
end;

procedure TDirectoryMenu.Loaded;
begin
inherited Loaded;
//Refresh;
end;

procedure TDirectoryMenu.SETFFileType(Value : TFileType);
begin
FFileType := Value;
Refresh;
end;

procedure TDirectoryMenu.mClick(Sender: TObject);
begin
  if Assigned(FOnFileMenuItemClick)
    then FOnFileMenuItemClick(Self, (Sender as TMenuItem).Hint);
end;

procedure TDirectoryMenu.SETFShowExt(Value : boolean);
begin
FShowExt := Value;
Refresh;
end;

procedure TDirectoryMenu.SETFDir(Value : string);
begin
FDir := Value;
if not FromSP then SP := GetSpecialFolder(Value);
Refresh;
end;

procedure TDirectoryMenu.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  case Operation of
    opInsert:
      begin

      end;
    opRemove:
      begin
      if DirMenuItem = AComponent then DirMenuItem := nil;
      end;
  end;
end;

procedure TDirectoryMenu.SETFDirMenuItem(Value : TMenuItem);
begin
   if FDirMenuItem <> Value then
   begin
   FDirMenuItem := Value;
   if Value <> nil then Value.FreeNotification(Self);
   Refresh;
   end;
end;

constructor TDirectoryMenu.Create(AOwner : TComponent);
begin
inherited Create(AOwner);
  FShowExt := False;
  FShowIcons := True;
  SP := spNone;
  FFileType := [ftReadOnly, ftDirectory, ftArchive, ftNormal];
  FImageList := TSystemImageList.Create(Self);
  ItemMenuTag := 12005;
  Filter := '*.*';
end;

destructor TDirectoryMenu.Destroy;
begin
if Assigned(FImageList) then FImageList.Free;
inherited Destroy;
end;

procedure TDirectoryMenu.Refresh;
begin
if csReading in Self.ComponentState then exit;
if csDesigning in Self.ComponentState then exit;
CreateItems;
if Assigned(OnRefreshed) then OnRefreshed(Self);
end;

procedure TDirectoryMenu.RemoveItems;
var I : Integer;
label 1;
begin
if not Assigned(FDirMenuItem) then exit;
if csReading in Self.ComponentState then exit;
if csDesigning in Self.ComponentState then exit;
1:
for I := 0 to FDirMenuItem.Count - 1 do
           begin
           if FDirMenuItem.Items[I].Tag = ItemMenuTag then begin FDirMenuItem.Delete(I); goto 1; end;
           end;
end;

function TDirectoryMenu.GN(FileName : string) : string;
begin
result := FileName;
if not FShowExt then Result := ChangeFileExt(FileName, '');
end;

procedure TDirectoryMenu.AddMenus(Item : TMenuItem; Dir : string);
var SR : TSearchRec;
    AttrIndex: TFileAttr;
    AttrWord: Word;
    MI : TMenuItem;
    ISDIR : Boolean;
    SaveCursor: TCursor;
    Include : boolean;
    Dirs, Files, SrcDirs : TStringList;
    I, J : Integer;
    DirA : string;
const
   Attributes: array[TFileAttr] of Word = (faReadOnly, faHidden, faSysFile,
     faVolumeID, faDirectory, faArchive, 0);
begin
try
Files := nil;
Dirs := nil;
SrcDirs := nil;
SrcDirs := TStringList.Create;
DirA := Dir;
if DirectoryExists(Dir) then SrcDirs.Add(IncludeTrailingPathDelimiter(DirA))
                        else
                        begin
                        while Pos(';', DirA) > 0 do
                             begin
                             SrcDirs.Add(IncludeTrailingPathDelimiter(Copy(DirA, 1, Pos(';', DirA) - 1)));
                             Delete(DirA, 1, Pos(';', DirA));
                             end;
                        SrcDirs.Add(IncludeTrailingPathDelimiter(DirA))     
                        end;
if not Assigned(Item) then exit;
Item.Tag := ItemMenuTag;
Dirs := TstringList.Create;
Files := TStringList.Create;
if ShowIcons then FDirMenuItem.SubMenuImages := FImageList
             else FDirMenuItem.SubMenuImages := nil;

    for AttrIndex := ftReadOnly to ftArchive do
      if AttrIndex in FileType then
        AttrWord := AttrWord or Attributes[AttrIndex];

  SaveCursor := Screen.Cursor;
  Screen.Cursor := crHourGlass;
  try
    for J := 0 to SrcDirs.Count - 1 do
      begin
      Dir := SrcDirs.Strings[J];
  if FindFirst(Dir + '*.*', AttrWord, sr) = 0 then
  repeat
    if (sr.Name[1] <> '.') then
    begin
    ISDIR := (sr.Attr and faDirectory) = faDirectory;
    Include := True;
    if ((Filter <> '*.*') or (Filter <> '')) and (not ISDIR) then
       begin
         if not WildcardMatch(SR.Name, FFilter) then Continue;
       end;
    if Assigned(OnIncludeItem) then OnIncludeItem(Self, Dir + SR.Name, ISDir, Include);
    if not Include then Continue;
    if ISDir then Dirs.Add(Dir + Sr.Name)
             else Files.Add(Dir + Sr.Name);
    end;
  until FindNext(sr) <> 0;
  FindClose(sr);
      end;
  Dirs.Sort;
  Files.Sort;
    for I := 0 to Dirs.Count - 1 do
    begin
    MI := NewItem(ExtractFileName(Dirs[I]),0,False,True,nil,0,'');
    MI.Tag := ItemMenuTag;
    if ShowIcons then MI.ImageIndex := FImageList.GetImageIndex({Dir + }Dirs[I]);
    MI.Hint := {Dir + }Dirs[I];
    if Item.Find(ExtractFileName(Dirs[I])) = nil then Item.Add(MI)
                                                 else MI := Item.Find(ExtractFileName(Dirs[I]));
    AddMenus(MI,IncludeTrailingPathDelimiter({Dir + }Dirs[I]));
    end;
    for I := 0 to Files.Count - 1 do
    begin
    MI := NewItem(GN(ExtractFileName(Files[I])),0,False,True,nil,0,'');
    MI.Tag := ItemMenuTag;
    if ShowIcons then MI.ImageIndex := FImageList.GetImageIndex({Dir + }Files[I]);
    MI.Hint := {Dir + }Files[I];
    if Item.Find(GN(ExtractFileName(Files[I]))) = nil then Item.Add(MI)
                                                 else
                                                 begin
                                                 MI := Item.Find(GN(ExtractFileName(Files[I])));
                                                 end;
    //Item.Add(MI);
    MI.OnClick := mClick;
    end;
  if Item.Count = 0 then Item.Add(NewItem(SEMPTY,0,False,False,nil,0,''));
  finally
  Screen.Cursor := SaveCursor;
  end;
finally
Dirs.Free;
Files.Free;
SrcDirs.Free;
end;
end;

procedure TDirectoryMenu.CreateItems;
begin
if csDesigning in Self.ComponentState then exit;
RemoveItems;
if not Assigned(FDirMenuItem) then exit;
if FDir = '' then Exit;
if not DirectoryExists(FDir) and (Pos(';', FDir) = 0) then Exit;
AddMenus(FDirMenuItem, FDir);
if (Pos(';', FDir) > 0) then QuickSort(FDirMenuItem);
end;

procedure TDirectoryMenu.QuickSort(AMenuItem: TMenuItem);
var I : Integer;
    Dirs, Files : TStringList;
begin
try
Dirs := TStringList.Create;
Files := TStringList.Create;
  for I := AMenuItem.Count - 1 downto 0 do
      begin
        if DirectoryExists(AMenuItem.Items[I].Hint) then Dirs.AddObject(AMenuItem.Items[I].Caption, AMenuItem.Items[I])
                                                    else if FileExists(AMenuItem.Items[I].Hint) or (AMenuItem.Count = 1) then Files.AddObject(AMenuItem.Items[I].Caption, AMenuItem.Items[I])
                                                                                                else if AMenuItem.Count > 1 then AMenuItem.Delete(I);
      end;
Dirs.Sort;
Files.Sort;
  for I := 0 to Files.Count - 1 do
      begin
        TMenuItem(Files.Objects[I]).MenuIndex := I;
        Files.Objects[I] := nil;
      end;
  for I := 0 to Dirs.Count - 1 do
      begin
        TMenuItem(Dirs.Objects[I]).MenuIndex := I;
        QuickSort(TMenuItem(Dirs.Objects[I]));
        Dirs.Objects[I] := nil;
      end;
finally
Dirs.Free;
Files.Free;
end;
end;

end.
