unit EM_Common;

{
  EasyMenus
  Copyright 2005 Murariu Alexandru
  muralexandru@yahoo.com
  All rights reserved.
  For conditions of distribution and use, see LICENSE.TXT.

}

interface
uses Windows, SysUtils, Classes, Types;

function WildcardMatch(Text, WildCard : string) : boolean;
function GetShellFolderPath(const FolderID: Integer): string;
function GetCommonShellFolder(ID, ID2 : Integer) : string;
function IsVersionOlder(OldVersion ,NewVersion : string) : boolean;

implementation
uses ShlObj, ActiveX;

type
  TFileVersionNumbers = record
      MS, LS: DWORD;
  end;

function AddPoint(const S: String): String; forward;
function PathLastChar(const S: String): PChar; forward;
function VersionNumbersToString(var VersionNumbers: TFileVersionNumbers) : string; forward;
function VersionNumbersToStringA(var VersionNumbers: TFileVersionNumbers) : string; forward;
function StringToVersionNumbers(STR  : string) : TFileVersionNumbers; forward;
function CheckVersionFormat(var Text : string) : boolean; forward;
function HowManyPoints(Text : string) : integer; forward;
function IsAcceptableVersion(var Text : string) : boolean; forward;

function GetShellFolderPath(const FolderID: Integer): String;
var pidl: PItemIDList;
    Buffer: array[0..MAX_PATH-1] of Char;
    Malloc: IMalloc;
begin
  Result := '';
  if FAILED(SHGetMalloc(Malloc)) then
    Malloc := nil;
    if SUCCEEDED(SHGetSpecialFolderLocation(0, FolderID, pidl)) then begin
    if SHGetPathFromIDList(pidl, Buffer) then
    Result := Buffer;
  if Assigned(Malloc) then
  Malloc.Free(pidl);
  end;
end;

function GetCommonShellFolder(ID, ID2 : Integer) : string;
begin
result := GetShellFolderPath(ID);
if result = '' then result := GetShellFolderPath(ID2);
end;

function ExtWCDS(s : string) : TStrings;
begin
  result := TStringList.Create;
  if Pos(';', s) = 0 then begin Result.Add(s); exit; end;
    repeat
    result.Add(Copy(S, 1, Pos(';', S) - 1));
    s := Copy(S, Pos(';', S) + 1, Length(S) - 2);
    if Pos(';', s) = 0 then Result.Add(s);
    until Pos(';', s) = 0;
end;

function WildcardMatchP(const Text, Pattern: PChar): Boolean;

  function CharCompare(const S1, S2: PChar): Boolean;
  var
    N, I: Integer;
  begin
    N := CharNext(S1) - S1;
    if N = CharNext(S2) - S2 then begin
      for I := 0 to N-1 do begin
        if S1[I] <> S2[I] then begin
          Result := False;
          Exit;
        end;
      end;
      Result := True;
    end else
      Result := False;
  end;

  function InternalWildcardMatch(Txt, Pat: PChar): Integer;
  begin
    while Pat^ <> #0 do begin
      case Pat^ of
        '?': ;
        '*': begin
               Inc(Pat);
               while Pat^ = '*' do begin
                 Inc(Pat);
               end;
               if Pat^ = #0 then begin
                 Result := 1;
                 Exit;
               end;
               while Txt^ <> #0 do begin
                 Result := InternalWildcardMatch(Txt, Pat);
                 if Result <> 0 then
                   Exit;
                 Txt := CharNext(Txt);
               end;
               Result := 2;
               Exit;
             end;
      else
        if not CharCompare(Txt, Pat) then begin
          Result := 0;
          Exit;
        end;
      end;
      Txt := CharNext(Txt);
      Pat := CharNext(Pat);
    end;
    if Txt^ = #0 then
      Result := 1
    else
      Result := 0;
  end;

begin
  Result := (InternalWildcardMatch(Text, Pattern) = 1);
end;

function WildcardMatch(Text, WildCard : string) : boolean;
var S : TStrings;
    I : Integer;
begin
  result := False;
  S := ExtWCDS(WildCard);
  for I := 0 to S.Count - 1 do
      begin
      if S.Strings[I] = '*.*' then begin result := True; exit; end;
      result := WildcardMatchP(PChar(LowerCase(Text)), PChar(LowerCase(S.Strings[I])));
      if result then exit;
      end;
end;

function PathLastChar(const S: String): PChar;
begin
  if S = '' then
    Result := nil
  else
    Result := CharPrev(Pointer(S), @S[Length(S)+1]);
end;

function AddPoint(const S: String): String;
begin
  Result := S;
  if (Result <> '') and (PathLastChar(Result)^ <> '.') then
    Result := Result + '.';
end;

function VersionNumbersToString(var VersionNumbers: TFileVersionNumbers) : string;
var ls : string;
begin
ls := IntToStr(HiWord(VersionNumbers.MS)) + '.' + IntToStr(LoWord(VersionNumbers.MS));
if (LoWord(VersionNumbers.LS)<>0) or (HiWord(VersionNumbers.LS)<>0) then ls :=ls + '.' + IntToStr(HiWord(VersionNumbers.LS)) + '.' + IntToStr(LoWord(VersionNumbers.LS));
result := LS;
end;

function VersionNumbersToStringA(var VersionNumbers: TFileVersionNumbers) : string;
var ls : string;
begin
ls := IntToStr(HiWord(VersionNumbers.MS)) + '.' + IntToStr(LoWord(VersionNumbers.MS));
ls :=ls + '.' + IntToStr(HiWord(VersionNumbers.LS)) + '.' + IntToStr(LoWord(VersionNumbers.LS));
result := LS;
end;

function StringToVersionNumbers(STR  : string) : TFileVersionNumbers;
var A,B : word;
begin
try
A := strtoint(Copy(STR,1,Pos('.',STR)-1));
Delete(STR,1,Pos('.',STR));
B := strtoint(Copy(STR,1,Pos('.',STR)-1));
Delete(STR,1,Pos('.',STR));
Result.MS := MakeLong(B,A);
A := strtoint(Copy(STR,1,Pos('.',STR)-1));
Delete(STR,1,Pos('.',STR));
B := strtoint(Copy(STR,1,Length(STR)));
Result.LS := MakeLong(B,A);
except
result.MS := MakeLong(0,0);
result.LS := MakeLong(0,0);
end;
end;

function IsVersionOlder(OldVersion ,NewVersion : string) : boolean;
var F,C : TFileVersionNumbers;
begin
try
IsAcceptableVersion(OldVersion);
IsAcceptableVersion(NewVersion);
CheckVersionFormat(OldVersion);
CheckVersionFormat(NewVersion);
F := StringToVersionNumbers(OldVersion);
C := StringToVersionNumbers(NewVersion);
result := not (HiWord(F.MS) = HiWord(C.MS));  // 1 < 0
if result then begin result := HiWord(F.MS) < HiWord(C.MS); exit; end;
result := not (LoWord(F.MS) = LoWord(C.MS));  // 0 < 8
if result then begin result := LoWord(F.MS) < LoWord(C.MS); exit; end;
result := not (HiWord(F.LS) = HiWord(C.LS));  // 0 < 0
if result then begin result := HiWord(F.LS) < HiWord(C.LS); exit; end;
result := not (LoWord(F.LS) = LoWord(C.LS));  // 2 < 8
if result then begin result := LoWord(F.LS) < LoWord(C.LS); exit; end;
except
result := false;
end;
end;

function HowManyPoints(Text : string) : integer;
var s:string;
label 1;
begin
s:=Text;
result := 0;
1:if Pos('.',S) > 0 then
                    begin
                    Delete(S,Pos('.',S),1);
                    inc(result);
                    goto 1;
                    end;
end;

function IsAcceptableVersion(var Text : string) : boolean;
var s:string;
    i:integer;
begin
result := true;
s:=Text;
for i:=0 to Length(s) - 1 do
      begin
      if Copy(s,i+1,1) <> '.' then
                     begin
                     try
                     strtoint(Copy(s,i+1,1))
                     except
                     result := false;
                     exit;
                     end;
                     end;
      end;
Text := s;
end;

function CheckVersionFormat(var Text : string) : boolean;
begin
result := true;
if HowManyPoints(TEXT) = 3 then
                    begin
                    result := true;
                    exit;
                    end;
if HowManyPoints(TEXT)>3 then result:=false
                  else
                  begin
                     repeat
                     Text := AddPoint(Text) + '0';
                     until HowManyPoints(TEXT) >= 3;
                  end;
end;

end.
