unit EasyMenus_Register;

{
  EasyMenus
  Copyright 2005 Murariu Alexandru
  muralexandru@yahoo.com
  All rights reserved.
  For conditions of distribution and use, see LICENSE.TXT.

}

interface
uses
  Windows, SysUtils, Classes,
  MultiLanguage, MRU, ToolsManager, ToolsManagerEditor, AlignedPopupMenu,
  Favorites, FavoritesEditor, SystemMenu, DirectoryMenu,
  SystemImageList, MenuStorage, DesignIntf, DesignEditors;

type
  TEasyMenuComponentEditor = class(TComponentEditor)
  public
    procedure ExecuteVerb(Index: Integer); override;
    function GetVerb(Index: Integer): String; override;
    function GetVerbCount: Integer; override;
  end;

  THKEYEditor = class(TPropertyEditor)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
  procedure SetValue(const Value: string); override;
  function GetValue : string; override;
  function GetAttributes: TPropertyAttributes; override;
  procedure GetValues(Proc: TGetStrProc); override;
    { Public declarations }
  published
    { Published declarations }
  end;

procedure Register;

implementation
uses EasyMenusAbout;

{$R AlignedPopupMenu.res}
{$R DirectoryMenu.res}
{$R Favorites.res}
{$R FavoritesEditor.res}
{$R MenuStorage.res}
{$R MRU.res}
{$R MultiLanguage.res}
{$R SystemMenu.res}
{$R ToolsManager.res}
{$R ToolsManagerEditor.res}

procedure Register;
begin
  RegisterComponents('Easy Menus', [TAlignedPopupMenu, TDirectoryMenu,
     TFavorites, TFavoritesEditor, TINIMenuStorage, TREGMenuStorage,
     TMRU, TMultiLanguage, TSystemImageList, TSystemMenu,
     TToolsManager, TToolsManagerEditor]);
//  RegisterComponentEditor(TAlignedPopupMenu, TEasyMenuComponentEditor);
  RegisterComponentEditor(TDirectoryMenu, TEasyMenuComponentEditor);
  RegisterComponentEditor(TFavorites, TEasyMenuComponentEditor);
  RegisterComponentEditor(TFavoritesEditor, TEasyMenuComponentEditor);
  RegisterComponentEditor(TINIMenuStorage, TEasyMenuComponentEditor);
  RegisterComponentEditor(TREGMenuStorage, TEasyMenuComponentEditor);
  RegisterComponentEditor(TMRU, TEasyMenuComponentEditor);
  RegisterComponentEditor(TMultiLanguage, TEasyMenuComponentEditor);
  RegisterComponentEditor(TSystemImageList, TEasyMenuComponentEditor);
//  RegisterComponentEditor(TSystemMenu, TEasyMenuComponentEditor);
  RegisterComponentEditor(TToolsManager, TEasyMenuComponentEditor);
  RegisterComponentEditor(TToolsManagerEditor, TEasyMenuComponentEditor);
  RegisterPropertyEditor(TypeInfo(Cardinal), TREGMenuStorage, 'Root', THKEYEditor);
end;

{ TEasyMenuComponentEditor }

procedure TEasyMenuComponentEditor.ExecuteVerb(Index: Integer);
begin
  case Index of
  0 : ShowAbout;
  else
  inherited;
  end;
end;

function TEasyMenuComponentEditor.GetVerb(Index: Integer): String;
begin
  case Index of
  0 : result := 'About...';
  else
  result := '';
  end;
end;

function TEasyMenuComponentEditor.GetVerbCount: Integer;
begin
  result := 1;
end;

procedure THKEYEditor.GetValues(Proc: TGetStrProc);
var I : Integer;
const
KeyNames : array[0..2] of string = ('HKEY_CURRENT_USER',
                                    'HKEY_USERS',
                                    'HKEY_LOCAL_MACHINE');
begin
for I := Low(KeyNames) to High(KeyNames) do Proc(KeyNames[I]);
end;

function THKEYEditor.GetValue : string;
begin
case GetOrdValue of
HKEY_CURRENT_USER : result := 'HKEY_CURRENT_USER';
HKEY_USERS : result := 'HKEY_USERS';
HKEY_LOCAL_MACHINE : result := 'HKEY_LOCAL_MACHINE';
else
result := 'Unknown Registry Key';
end;
end;

procedure THKEYEditor.SetValue(const Value: string);
begin
if SameText(Value,'HKEY_CURRENT_USER') then SetOrdValue(HKEY_CURRENT_USER);
if SameText(Value,'HKEY_USERS') then SetOrdValue(HKEY_USERS);
if SameText(Value,'HKEY_LOCAL_MACHINE') then SetOrdValue(HKEY_LOCAL_MACHINE);
end;

function THKEYEditor.GetAttributes: TPropertyAttributes;
begin
 Result := [paMultiSelect, paValueList];
end;

end.
