unit Favorites;

{
  EasyMenus
  Copyright 2005 Murariu Alexandru
  muralexandru@yahoo.com
  All rights reserved.
  For conditions of distribution and use, see LICENSE.TXT.

}

interface

uses
  Windows, Messages, SysUtils, Classes, Menus, MenuStorage;

type

  TFavoritesItemClick = procedure (Sender : TObject; Favorite : String) of Object;

  TFavorites = class(TComponent)
  private
    FFavorites : TStrings;
    FFavoritesMenuItem : TMenuItem;
    FFavoritesItemClick : TFavoritesItemClick;
    FMenuStorage : TMenuStorage;
    FAutoLoad : boolean;
    FOnRefreshed: TNotifyEvent;
    procedure SetFFavoritesMenuItem(Value : TMenuItem);
    procedure SETFFavorites(Value : TStrings);
    function GetFAVCaption(Index : integer; DefCap : string) : string;
    procedure OnItemClick(Sender : TObject);
    procedure SETFMenuStorage(Value : TMenuStorage);
    function GETCount : integer;
    procedure RemoveItems;
    { Private declarations }
  protected
    procedure Notification(AComponent: TComponent;
    Operation: TOperation); override;
    procedure Loaded; override;
    { Protected declarations }
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Refresh;
    procedure Save;
    procedure Load;
    procedure Add(Favorite : string);
    procedure Delete(Index : integer);
    procedure Clear;
    property Count : integer read GETCount;
    { Public declarations }
  published
    property AutoLoad : boolean read FAutoLoad write FAutoLoad;
    property MenuStorage : TMenuStorage read FMenuStorage write SETFMenuStorage;
    property Favorites : TStrings read FFavorites write SETFFavorites;
    property FavoritesMenuItem : TMenuItem read FFavoritesMenuItem write SetFFavoritesMenuItem;
    property OnFavoritesItemClick : TFavoritesItemClick read FFavoritesItemClick write FFavoritesItemClick;
    property OnRefreshed : TNotifyEvent read FOnRefreshed write FOnRefreshed;
    { Published declarations }
  end;

implementation

procedure TFavorites.Loaded;
begin
inherited Loaded;
if AutoLoad then Load;
end;

procedure TFavorites.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  case Operation of
    opInsert:
      begin
      if (AComponent is TMenuStorage) and (MenuStorage = nil) then MenuStorage := (AComponent as TMenuStorage);
      end;
    opRemove:
      begin
      if FavoritesMenuItem = AComponent then FavoritesMenuItem := nil;
      if MenuStorage = AComponent then MenuStorage := nil;
      end;
  end;
end;

procedure TFavorites.Save;
var I : Integer;
begin
if not Assigned(FMenuStorage) then Exit;
FMenuStorage.WriteInteger('Favorites_Count',FFavorites.Count);
for i := 0 to FFavorites.Count - 1 do
    begin
    FMenuStorage.WriteString('Favorite'+IntToStr(I),FFavorites.Strings[i]);
    end;
end;

procedure TFavorites.Load;
var I : Integer;
    S : String;
begin
if not Assigned(FMenuStorage) then Exit;
FFavorites.Clear;
for i := 0 to FMenuStorage.ReadInteger('Favorites_Count',0,0,0) - 1 do
    begin
    S := FMenuStorage.ReadString('Favorite'+IntToStr(I),'');
    if S <> '' then FFavorites.Add(S);
    end;
Refresh;
end;

procedure TFavorites.Add(Favorite : string);
begin
if Favorite = '' then exit;
FFavorites.Add(Favorite);
Refresh;
end;

procedure TFavorites.Delete(Index : integer);
begin
FFavorites.Delete(Index);
Refresh;
end;

procedure TFavorites.Clear;
begin
FFavorites.Clear;
Refresh;
end;

function TFavorites.GETCount : integer;
begin
result := FFavorites.Count;
end;

constructor TFavorites.Create(AOwner : TComponent);
begin
inherited Create(AOwner);
FFavorites := TStringList.Create;
FAutoLoad := True;
end;

destructor TFavorites.Destroy;
begin
RemoveItems;
if Assigned(FFavorites) then FFavorites.Free;
inherited Destroy;
end;

procedure TFavorites.SETFFavorites(Value : TStrings);
begin
FFavorites.Assign(Value);
Refresh;
end;

procedure TFavorites.SETFMenuStorage(Value : TMenuStorage);
begin
  if FMenuStorage <> Value then
  begin
    FMenuStorage := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TFavorites.SetFFavoritesMenuItem(Value : TMenuItem);
begin
  if FFavoritesMenuItem <> Value then
  begin
    RemoveItems;
    FFavoritesMenuItem := Value;
    if Value <> nil then Value.FreeNotification(Self);
    Refresh;
  end;
end;

function TFavorites.GetFAVCaption(Index : integer; DefCap : string) : string;
begin
case index of
0..9 : result := '&' + inttostr(index) + '  ' + DefCap;
Ord('A')-65+10..Ord('Z')-65+10 : result := '&' + Chr(65+index-10) + '  ' + DefCap;
else
result := '   ' + DefCap;
end;
end;

procedure TFavorites.OnItemClick(Sender : TObject);
begin
if Assigned(OnFavoritesItemClick) then OnFavoritesItemClick(Self, FFavorites.Strings[(Sender as TMenuItem).Tag - 1]);
end;

procedure TFavorites.RemoveItems;
var I : Integer;
label 1;
begin
if not Assigned(FFavoritesMenuItem) then exit;
1:for I := 0 to FFavoritesMenuItem.Count - 1 do
    if FFavoritesMenuItem.Items[I].Tag <> 0 then
               begin
               FFavoritesMenuItem.Items[i].Free;
               goto 1;
               end;
end;

procedure TFavorites.Refresh;
var I : integer;
    MI : TMenuItem;
begin
if csDesigning in Self.ComponentState then exit;
RemoveItems;
if not Assigned(FFavoritesMenuItem) then exit;
if FFavorites.Count = 0 then exit;
for I := 0 to FFavorites.Count - 1 do
    begin
    MI := TMenuItem.Create(nil);
    MI.Caption := GetFAVCaption(I{ + 1}, FFavorites.Strings[i]);
    MI.Tag := I + 1;
    MI.OnClick := OnItemClick;
    FFAvoritesMenuItem.Add(MI);
    end;
if Assigned(OnRefreshed) then OnRefreshed(Self);
end;

end.
 