unit FavoritesEditor;

{
  EasyMenus
  Copyright 2005 Murariu Alexandru
  muralexandru@yahoo.com
  All rights reserved.
  For conditions of distribution and use, see LICENSE.TXT.

}

interface

uses
  Windows, Messages, SysUtils, Classes, Favorites, ComCtrls, Controls;

type
  TFavoritesEditor = class(TComponent)
  private
  FFavorites : TFavorites;
  FHelpType : THelpType;
  FHelpKeyword : String;
  FHelpContext : THelpContext;
  procedure SetHelpContext(const Value: THelpContext);
  procedure SetHelpKeyword(const Value: String);
    { Private declarations }
  protected
  procedure Notification(AComponent: TComponent;
  Operation: TOperation); override;
    { Protected declarations }
  public
  function Edit : boolean;
  constructor Create(AOwner: TComponent); override;
    { Public declarations }
  published
  property HelpType : THelpType read FHelpType write FHelpType;// default htContext;
  property HelpKeyword : string read FHelpKeyword write SetHelpKeyword stored True;
  property HelpContext : THelpContext read FHelpContext write SetHelpContext stored True default 0;
  property Favorites : TFavorites read FFavorites write FFavorites;
    { Published declarations }
  end;

implementation
uses FavoritesEditorForm_unit, Forms;

procedure TFavoritesEditor.SetHelpContext(const Value: THelpContext);
begin
  if not (csLoading in ComponentState) then FHelpType := htContext;
  FHelpContext := Value;
end;

procedure TFavoritesEditor.SetHelpKeyword(const Value: String);
begin
  if not (csLoading in ComponentState) then FHelpType := htKeyword;
  FHelpKeyword := Value;
end;

procedure TFavoritesEditor.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  case Operation of
    opInsert:
      begin
      if (AComponent is TFavorites) and (Favorites = nil) then Favorites := (AComponent as TFavorites);
      end;
    opRemove:
      begin
      if Favorites = AComponent then Favorites := nil;
      end;
  end;
end;

function TFavoritesEditor.Edit : boolean;
var FavoritesEditorForm: TFavoritesEditorForm;
    I : Integer;
    L : TListItem;
    TL : TStringList;
    S : String;
begin
result := False;
if not Assigned(FFavorites) then exit;
FavoritesEditorForm := TFavoritesEditorForm.Create(nil);
FavoritesEditorForm.HelpKeyword := HelpKeyword;
FavoritesEditorForm.HelpContext := HelpContext;
FavoritesEditorForm.HelpType := HelpType;
FavoritesEditorForm.ListView1.Clear;
for I := 0 to FFavorites.Favorites.Count - 1 do
    begin
    with FavoritesEditorForm do
        begin
        L := ListView1.Items.Add;
        L.Caption := FFavorites.Favorites.Strings[i];
        end;
    end;
FavoritesEditorForm.Position := poScreenCenter;
if FavoritesEditorForm.ShowModal <> mrok then
                                         begin
                                         FavoritesEditorForm.Free;
                                         Exit;
                                         end;
TL := TStringList.Create;
for I := 0 to FavoritesEditorForm.ListView1.Items.Count - 1 do
             begin
             S := FavoritesEditorForm.ListView1.Items.Item[I].Caption;
             if S <> '' then TL.Add(S);
             end;
FFavorites.Favorites.Assign(TL);
TL.Free;
FFavorites.Refresh;
result := True;
end;

constructor TFavoritesEditor.Create(AOwner: TComponent);
begin
  inherited;
  HelpType := htContext;
end;

end.
