unit MRU;

{
  EasyMenus
  Copyright 2005 Murariu Alexandru
  muralexandru@yahoo.com
  All rights reserved.
  For conditions of distribution and use, see LICENSE.TXT.

}

interface

uses
  Windows, Messages, SysUtils, Classes, Menus, Dialogs, MenuStorage;

type

  TMRUItemClick = procedure (Sender : TObject; Item : string) of Object;

  TMRU = class(TComponent)
  private
    FMRU : TStringList;
    FMAXitems : integer;
    FMRUMenuItem : TMenuItem;
    ICLICK : TMRUItemClick;
    FMenuStorage : TMenuStorage;
    FAutoLoad : boolean;
    FOnRefreshed: TNotifyEvent;
    procedure SETFMAXItems(Value : integer);
    procedure SetFMRUMenuItem(Value : TMenuItem);
    procedure OnItemClick(Sender : TObject);
    function GetMRUCaption(Index : integer; DefCap : string) : string;
    function Get(Index: Integer): string;
    procedure Put(Index: Integer; Item: string);
    function ItemsCount : integer;
    procedure SETFMenuStorage(Value : TMenuStorage);
    procedure RemoveItems;
    { Private declarations }
  protected
    procedure Notification(AComponent: TComponent;
    Operation: TOperation); override;
    procedure Loaded; override;
    { Protected declarations }
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Refresh;
    procedure Add(Item : string);
    procedure Clear;
    property Items[Index : integer]: string read Get write Put; default;
    property Count : integer read ItemsCount;
    procedure Save;
    procedure Load;
    { Public declarations }
  published
    property AutoLoad : boolean read FAutoLoad write FAutoLoad;
    property MenuStorage : TMenuStorage read FMenuStorage write SETFMenuStorage;
    property MaxItems : integer read FMAXitems write SETFMAXItems;
    property MRUMenuItem : TMenuItem read FMRUMenuItem write SetFMRUMenuItem;
    property OnMRUItemClick : TMRUItemClick read ICLICK write ICLICK;
    property OnRefreshed : TNotifyEvent read FOnRefreshed write FOnRefreshed;
    { Published declarations }
  end;

implementation

procedure TMRU.Loaded;
begin
inherited Loaded;
if AutoLoad then Load;
end;

procedure TMRU.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  case Operation of
    opInsert:
      begin
      if (AComponent is TMenuStorage) and (MenuStorage = nil) then MenuStorage := (AComponent as TMenuStorage);
      end;
    opRemove:
      begin
      if MRUMenuItem = AComponent then MRUMenuItem := nil;
      if MenuStorage = AComponent then MenuStorage := nil;
      end;
  end;
end;

procedure TMRU.Save;
var I : Integer;
begin
if not Assigned(FMenuStorage) then Exit;
FMenuStorage.WriteInteger('MRU_Count',Count);
for i := 0 to Count - 1 do
    begin
    FMenuStorage.WriteString('MRU'+IntToStr(I),Items[i]);
    end;
end;

procedure TMRU.Load;
var I, Z : Integer;
    S : String;
begin
if not Assigned(FMenuStorage) then Exit;
Clear;
Z := FMenuStorage.ReadInteger('MRU_Count',0,0,0);
if Z > FMAXitems then Z := FMAXitems;
for i := 0 to Z - 1 do
    begin
    S := FMenuStorage.ReadString('MRU'+IntToStr(I),'');
    if S <> '' then FMRU.Add(S);
    end;
Refresh;
end;

constructor TMRU.Create(AOwner : TComponent);
begin
inherited Create(AOwner);
FMRU := TStringList.Create;
FMRU.CaseSensitive := False;
FMAXitems := 15;
FAutoLoad := True;
end;

destructor TMRU.Destroy;
begin
RemoveItems;
if Assigned(FMRU) then FMRU.Free;
inherited Destroy;
end;

function TMRU.Get(Index: Integer): string;
begin
if (index > FMAXitems - 1) or (index > FMRU.Count - 1) then exit;
result := FMRU.Strings[index];
end;

procedure TMRU.Put(Index: Integer; Item: string);
begin
if (index > FMAXitems - 1) or (index > FMRU.Count - 1) then exit;
FMRU.Strings[index] := Item;
end;

function TMRU.ItemsCount : integer;
begin
result := FMRU.Count;
if result > FMAXitems then result := FMAXitems;
end;

procedure TMRU.OnItemClick(Sender : TObject);
begin
if Assigned(OnMRUItemClick) then OnMRUItemClick(Self, FMRU.Strings[(Sender as TMenuItem).Tag - 1]);
Add(FMRU.Strings[(Sender as TMenuItem).Tag - 1]);
end;

procedure TMRU.Add(Item : string);
label 1;
begin
if Item = '' then exit;
case FMRU.IndexOf(Item) of
-1 : FMRU.Insert(0,Item);
else
FMRU.Move(FMRU.IndexOf(Item),0);
end;
1: if FMRU.Count > FMAXitems then
      begin
      FMRU.Delete(FMRU.Count - 1);
      goto 1;
      end;
Refresh;
end;

procedure TMRU.Clear;
begin
FMRU.Clear;
Refresh;
end;

procedure TMRU.SETFMenuStorage(Value : TMenuStorage);
begin
  if FMenuStorage <> Value then
  begin
    FMenuStorage := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TMRU.SetFMRUMenuItem(Value : TMenuItem);
begin
  if FMRUMenuItem <> Value then
  begin
    RemoveItems;
    FMRUMenuItem := Value;
    if Value <> nil then Value.FreeNotification(Self);
    Refresh;
  end;
end;

function TMRU.GetMRUCaption(Index : integer; DefCap : string) : string;
begin
case index of
0..9 : result := '&' + inttostr(index) + '  ' + DefCap;
Ord('A')-65+10..Ord('Z')-65+10 : result := '&' + Chr(65+index-10) + '  ' + DefCap;
else
result := '   ' + DefCap;
end;
end;

procedure TMRU.RemoveItems;
var I : Integer;
label 1;
begin
if not Assigned(FMRUMenuItem) then exit;
1:for I := 0 to FMRUMenuItem.Count - 1 do
    if FMRUMenuItem.Items[I].Tag <> 0 then
               begin
               FMRUMenuItem.Items[i].Free;
               goto 1;
               end;
end;

procedure TMRU.Refresh;
var I,C : integer;
    MI : TMenuItem;
begin
if csDesigning in Self.ComponentState then exit;
RemoveItems;
if not Assigned(FMRUMenuItem) then exit;
if FMRU.Count = 0 then exit;
C := FMRU.Count;
if C > FMAXitems then C := FMAXitems;
for I := 0 to C - 1 do
    begin
    MI := TMenuItem.Create(nil);
    MI.Caption := GetMRUCaption(I{ + 1}, FMRU.Strings[i]);
    MI.Tag := I + 1;
    MI.OnClick := OnItemClick;
    FMRUMenuItem.Insert(I,MI);
    end;
if Assigned(OnRefreshed) then OnRefreshed(Self);
end;

procedure TMRU.SETFMAXItems(Value : integer);
begin
FMAXitems := Value;
Refresh;
end;

end.
