unit MenuStorage;

{
  EasyMenus
  Copyright 2005 Murariu Alexandru
  muralexandru@yahoo.com
  All rights reserved.
  For conditions of distribution and use, see LICENSE.TXT.

}

interface

uses
  Windows, Messages, SysUtils, Classes, Registry;

type
  TMenuStorage = class(TComponent)
  private
    { Private declarations }
  protected
    { Protected declarations }
  public
  procedure WriteString(const Name, Value : string); virtual;
  procedure WriteInteger(const Name : string; const Value : Longint); virtual;
  function ReadInteger(const Name : string; const Default, Min, Max : Longint) : Longint; virtual;
  function ReadString(const Name, Default : String) : string; virtual;
    { Public declarations }
  published
    { Published declarations }
  end;

  TINIMenuStorage = class(TMenuStorage)
  private
  FFile : TFileName;
  FSection : String;
    { Private declarations }
  protected
    { Protected declarations }
  public
  procedure WriteString(const Name, Value : string); override;
  procedure WriteInteger(const Name : string; const Value : Longint); override;
  function ReadInteger(const Name : string; const Default, Min, Max : Longint) : Longint; override;
  function ReadString(const Name, Default : String) : string; override;
    { Public declarations }
  published
  property FileName : TFileName read FFile write FFile;
  property Section : String read FSection write FSection;
    { Published declarations }
  end;

  TREGMenuStorage = class(TMenuStorage)
  private
  FSubKey : String;
  FKEY : Cardinal;
    { Private declarations }
  protected
    { Protected declarations }
  public
  procedure WriteString(const Name, Value : string); override;
  procedure WriteInteger(const Name : string; const Value : Longint); override;
  function ReadInteger(const Name : string; const Default, Min, Max : Longint) : Longint; override;
  function ReadString(const Name, Default : String) : string; override;
  constructor Create(AOwner : TComponent); override;
    { Public declarations }
  published
  property Root : Cardinal read FKEY write FKEY;
  property SubKey : String read FSubKey write FSubKey;
    { Published declarations }
  end;

implementation

procedure TMenuStorage.WriteString(const Name, Value : string);
begin

end;

procedure TMenuStorage.WriteInteger(const Name : string; const Value : Longint);
begin

end;

function TMEnuStorage.ReadInteger(const Name : string; const Default, Min, Max : Longint) : Longint;
begin
result := 0;
end;

function TMenuStorage.ReadString(const Name, Default : String) : string;
begin
result := '';
end;

{INIMenuStorage}

procedure TINIMenuStorage.WriteString(const Name, Value : string);
begin
if FFile = '' then Exit;
WritePrivateProfileString(PChar(FSection), PChar(Name), PChar(Value), PChar(FFile));
end;

procedure TINIMenuStorage.WriteInteger(const Name : string; const Value : Longint);
begin
WriteString(Name, IntToStr(Value));
end;

function TINIMenuStorage.ReadInteger(const Name : string; const Default, Min, Max : Longint) : Longint;
begin
result := StrToIntDef(ReadString(Name,IntToStr(Default)),Default);
if (result < Min) and (Min <> 0) and (Max <> 0) then Result := min;
if (result > Max) and (Min <> 0) and (Max <> 0) then result := max;
end;

function TINIMenuStorage.ReadString(const Name, Default : String) : string;
var
  Buffer: array[0..2047] of Char;
begin
  SetString(Result, Buffer, GetPrivateProfileString(PChar(FSection),
    PChar(Name), PChar(Default), Buffer, SizeOf(Buffer), PChar(FFile)));
end;

{REGMenuStorage}

procedure TREGMenuStorage.WriteString(const Name, Value : string);
var REG : TRegistry;
begin
try
REG := TRegistry.Create;
REG.RootKey := FKEY;
REG.OpenKey(SubKey,True);
REG.WriteString(Name,Value);
REG.CloseKey;
reg.Free;
except
FreeAndNil(REG);
end;
end;

procedure TREGMenuStorage.WriteInteger(const Name : string; const Value : Longint);
begin
WriteString(Name, IntToStr(Value));
end;

function TREGMenuStorage.ReadInteger(const Name : string; const Default, Min, Max : Longint) : Longint;
begin
result := StrToIntDef(ReadString(Name,IntToStr(Default)),Default);
if (result < Min) and (Min <> 0) and (Max <> 0) then Result := min;
if (result > Max) and (Min <> 0) and (Max <> 0) then result := max;
end;

function TREGMenuStorage.ReadString(const Name, Default : String) : string;
var REG : TRegistry;
begin
try
result := Default;
REG := TRegistry.Create;
REG.RootKey := FKEY;
REG.OpenKey(SubKey,True);
result := REG.ReadString(Name);
if result = '' then result := Default;
REG.CloseKey;
reg.Free;
except
FreeAndNil(REG);
end;
end;

constructor TREGMenuStorage.Create(AOwner : TComponent);
begin
inherited Create(AOwner);
FKEY := HKEY_CURRENT_USER;
end;

end.
