unit SystemImageList;

{
  EasyMenus
  Copyright 2005 Murariu Alexandru
  muralexandru@yahoo.com
  All rights reserved.
  For conditions of distribution and use, see LICENSE.TXT.

}

interface

uses
  Windows, Messages, SysUtils, Classes, ImgList, Controls, Graphics;

type

  TSysIconType = (itSmallIcon, itLargeIcon);

  TSystemImageList = class(TImageList)
  private
    FIconType: TSysIconType;
    procedure RecreateImgListHandle;
    procedure SetFIconType(const Value: TSysIconType);
    { Private declarations }
  protected
    { Protected declarations }
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    function GetImageIndex(const AFile: string): integer;
    function GetFileIcon(const AFile : string; AddIconLinkOverlay : Boolean; out Icon : TIcon) : boolean; overload;
    function GetFileIcon(const AFile : string; AddIconLinkOverlay : Boolean; IconType : TSysIconType; out Icon : TIcon) : boolean; reintroduce; overload;
    function GetFileIcon(const AFile : string; out Icon : TIcon) : boolean; reintroduce; overload;
    function GetFileIcon(const AFile : string; IconType : TSysIconType; out Icon : TIcon) : boolean; reintroduce; overload;
    { Public declarations }
  published
    property IconType : TSysIconType read FIconType write SetFIconType;
    { Published declarations }
  end;

implementation
uses ShellApi;

   const
   SysImgListFlags : array[TSysIconType] of Cardinal = (SHGFI_SMALLICON, SHGFI_LARGEICON);
   SysIconLinkOverlay : array[Boolean] of Cardinal = (0, SHGFI_LINKOVERLAY);

function TSystemImageList.GetImageIndex(const AFile: string): integer;
var
  Info: TSHFileInfo;
begin
  if SHGetFileInfo(PChar(AFile), 0, Info, SizeOf(TSHFileInfo),
      SysImgListFlags[FIconType] or SHGFI_SYSICONINDEX) <> 0
    then Result := Info.IIcon
    else Result := 0;
end;

constructor TSystemImageList.Create(AOwner : TComponent);
begin
inherited Create(AOwner);
FIconType := itSmallIcon;
RecreateImgListHandle;
end;

destructor TSystemImageList.Destroy;
begin
inherited Destroy;
end;

procedure TSystemImageList.RecreateImgListHandle;
var
  himl: DWORD;
  Info: TSHFileInfo;
begin
if not (csDesigning in Self.ComponentState) then
begin
  Width := 16;
  Height := 16;
  himl := SHGetFileInfo('', 0, Info, SizeOf(TSHFileInfo),
    SysImgListFlags[FIconType] or SHGFI_SYSICONINDEX);
  if (himl <> 0) then
  begin
    Handle := himl;
    ShareImages := True;
    DrawingStyle := dsTransparent;
  end;
end;
end;

procedure TSystemImageList.SetFIconType(const Value: TSysIconType);
begin
  FIconType := Value;
  RecreateImgListHandle;
end;

function TSystemImageList.GetFileIcon(const AFile: string;
  AddIconLinkOverlay: Boolean; out Icon: TIcon): boolean;
begin
  result := GetFileIcon(AFile, AddIconLinkOverlay, FIconType, Icon);
end;

function TSystemImageList.GetFileIcon(const AFile: string;
  AddIconLinkOverlay: Boolean; IconType: TSysIconType;
  out Icon: TIcon): boolean;
var
  Info: TSHFileInfo;
begin
Icon := TIcon.Create;
  result :=  SHGetFileInfo(PChar(AFile), 0, Info, SizeOf(TSHFileInfo),
      SysImgListFlags[IconType] or SHGFI_SYSICONINDEX or SHGFI_ICON or SysIconLinkOverlay[AddIconLinkOverlay]) <> 0;
  if result then Icon.Handle := Info.hIcon;
end;

function TSystemImageList.GetFileIcon(const AFile: string;
  out Icon: TIcon): boolean;
begin
  result := GetFileIcon(AFile, False, Icon);
end;

function TSystemImageList.GetFileIcon(const AFile: string;
  IconType: TSysIconType; out Icon: TIcon): boolean;
begin
  result := GetFileIcon(AFile, False, IconType, Icon);
end;

end.
