unit SystemMenu;

{
  EasyMenus
  Copyright 2005 Murariu Alexandru
  muralexandru@yahoo.com
  All rights reserved.
  For conditions of distribution and use, see LICENSE.TXT.

  TSystemMenu Component
  SOME FEATURES:
   - With this component you can add your own menu items into the system menu of your Form.
   - Edit the TSystemMenu with the Menu Editor.
   - Add items, assign OnClick events, etc.
   - The properties of TMenuItem are all supported, even the images are drawed on the system menu.
  NOTE: You can only have one TSystemMenu per Form.

}

interface

uses
  Windows, Messages, SysUtils, Classes, Forms,
  Controls, Dialogs, Graphics, Menus, Imglist;

type

  TSystemMenu = class(TPopupMenu)
  private
  FActive : Boolean;
  smOldFormWndProc: TWndMethod;
  FONPOP : TNotifyEvent;
    FDesPreview: boolean;
  procedure SETFActive(Value : Boolean);
  procedure CreateItems;
  procedure RemoveItems;
    { Private declarations }
  protected
  procedure smNewFormWindowProc(var Message: TMessage);
  procedure CallClickHandler(AMenuItem: TMenuItem; aCommand: Integer);
    { Protected declarations }
  public
  procedure Refresh;
  constructor Create(AOwner : TComponent); override;
  destructor Destroy; override;
    { Public declarations }
  published
  property DesignPreview : boolean read FDesPreview write FDesPreview;
  property Active : boolean read FActive write SETFActive;
  property OnSysPopup : TNotifyEvent read FONPOP write FONPOP;
    { Published declarations }
  end;

implementation

procedure DoDrawText(MenuItem : TMenuItem; ACanvas: TCanvas; const ACaption: string;
  var Rect: TRect; Selected: Boolean; Flags: Longint);
var
  Text: string;
  R: TRect;
  ParentMenu: TMenu;
begin
with MenuItem do
begin
  ParentMenu := GetParentMenu;
  if (ParentMenu <> nil) and (ParentMenu.IsRightToLeft) then
  begin
    if Flags and DT_LEFT = DT_LEFT then
      Flags := Flags and (not DT_LEFT) or DT_RIGHT
    else if Flags and DT_RIGHT = DT_RIGHT then
      Flags := Flags and (not DT_RIGHT) or DT_LEFT;
    Flags := Flags or DT_RTLREADING;
  end;
  Text := ACaption;
  if (Flags and DT_CALCRECT <> 0) and ((Text = '') or
    (Text[1] = cHotkeyPrefix) and (Text[2] = #0)) then Text := Text + ' ';
  with ACanvas do
  begin
    if Text = cLineCaption then
    begin
      if Flags and DT_CALCRECT = 0 then
      begin
        R := Rect;
        Inc(R.Top, 4);
        DrawEdge(Handle, R, EDGE_ETCHED, BF_TOP);
      end;
    end
    else
    begin
      Brush.Style := bsClear;
      if Default then
        Font.Style := Font.Style + [fsBold];
      if not Enabled then
      begin
        if not Selected then
        begin
          OffsetRect(Rect, 1, 1);
          Font.Color := clBtnHighlight;
          DrawText(Handle, PChar(Text), Length(Text), Rect, Flags);
          OffsetRect(Rect, -1, -1);
        end;
        if Selected and (ColorToRGB(clHighlight) = ColorToRGB(clBtnShadow)) then
          Font.Color := clBtnHighlight else
          Font.Color := clBtnShadow;
      end;
      DrawText(Handle, PChar(Text), Length(Text), Rect, Flags);
    end;
  end;
end;
end;

procedure MeasureItem(MenuItem : TMenuItem; ACanvas: TCanvas; var Width, Height: Integer);
const
  Alignments: array[TPopupAlignment] of Word = (DT_LEFT, DT_RIGHT, DT_CENTER);
var
  Alignment: TPopupAlignment;
  ImageList: TCustomImageList;
  ParentMenu: TMenu;
  DrawGlyph: Boolean;
  TopLevel: Boolean;
  DrawStyle: Integer;
  Text: string;
  R: TRect;

  procedure GetMenuSize;
  var
    NonClientMetrics: TNonClientMetrics;
  begin
    NonClientMetrics.cbSize := sizeof(NonClientMetrics);
    if SystemParametersInfo(SPI_GETNONCLIENTMETRICS, 0, @NonClientMetrics, 0) then
    begin
      Width := NonClientMetrics.iMenuWidth;
      Height := NonClientMetrics.iMenuHeight;
    end;
  end;

begin
with MenuItem do
begin
  if GetParentComponent is TMainMenu then
  begin
    TopLevel := True;
    GetMenuSize;
  end
  else TopLevel := False;
  ParentMenu := GetParentMenu;
  ImageList := GetImageList;
  if Caption = cLineCaption then
  begin
    Height := 5;
    Width := -2;
    DrawGlyph := False;
  end
  else if Assigned(ImageList) and ((ImageIndex > -1) or not TopLevel) then
  begin
    Width := ImageList.Width;
    if not TopLevel then
      Height := ImageList.Height;
    DrawGlyph := True;
  end
  else if Assigned(Bitmap) and not Bitmap.Empty then
  begin
    Width := 16;
    if not TopLevel then
      Height := 16;
    DrawGlyph := True;
  end
  else
  begin
    Width := -7;
    DrawGlyph := False;
  end;
  if DrawGlyph and not TopLevel then
    Inc(Width, 15);
  if not TopLevel then
    Inc(Height, 3);
  FillChar(R, SizeOf(R), 0);
  if ParentMenu is TMenu then
    Alignment := paLeft
  else if ParentMenu is TPopupMenu then
    Alignment := TPopupMenu(ParentMenu).Alignment
  else
    Alignment := paLeft;
  if ShortCut <> 0 then
    Text := Concat(Caption, ShortCutToText(ShortCut)) else
    Text := Caption;
  DrawStyle := Alignments[Alignment] or DT_EXPANDTABS or DT_SINGLELINE or
    DT_NOCLIP or DT_CALCRECT;
  DoDrawText(MenuItem ,ACanvas, Text, R, False, DrawStyle);
  Inc(Width, R.Right - R.Left + 7);
  if Assigned(OnMeasureItem) then OnMeasureItem(MenuItem, ACanvas, Width, Height);
end;
end;

procedure TSystemMenu.SETFActive(Value : Boolean);
begin
FActive := Value;
Refresh;
end;

procedure TSystemMenu.Refresh;
begin
CreateItems;
end;

procedure TSystemMenu.CreateItems;
var SYSMenu : HMENU;
    I, Flags : integer;
    Caption : String;
begin
if not Assigned(Owner) then exit;
RemoveItems;
if not FActive then Exit;
if Items.Count = 0 then exit;
SYSMenu := GetsystemMenu((Owner as TCustomForm).Handle, False);
AppendMenu(SysMenu, MF_SEPARATOR, $E000, '');
for I := 0 to Items.count-1 do
    begin
      Flags := MF_STRING;
      if Items.Items[I].Caption = '-' then Flags := MF_SEPARATOR;
      if Items.Items[I].Checked then
      begin
        Flags := Flags or MF_CHECKED;
        if Items.Items[I].RadioItem then Flags := Flags or MFT_RADIOCHECK;
      end;
      if not Items.Items[I].Enabled then Flags := Flags or MF_GRAYED;
      if Items.Items[I].Break = mbBarBreak then Flags := Flags or MF_MENUBARBREAK;
      if Items.Items[I].Break = mbBreak then Flags := Flags or MF_MENUBREAK;
      Caption := Items.Items[I].Caption;
      if Items.Items[I].ShortCut <> 0 then
      begin
        Caption := Caption + #9 +
          ShortCutToText(Items.Items[I].ShortCut);
      end;
      if Items.Items[I].count <> 0 then
      begin
        AppendMenu(SysMenu, MF_POPUP,
                   integer(Items.Items[I].Handle),
                   Pchar(Items.Items[I].Caption));
      end
      else
      begin
        if Assigned(Images) or (not Items.Items[I].Bitmap.Empty) then FLAGS := FLAGS or MF_OWNERDRAW;
        AppendMenu(SysMenu, FLAGS, Items.Items[I].Command, Pchar(Caption));
      end;
    end;
end;

procedure TSystemMenu.RemoveItems;
var SYSMenu : HMENU;
    PP : integer;
const
IDSARR : array[0..7] of integer = (SC_CLOSE, SC_MINIMIZE, SC_MAXIMIZE, SC_SIZE, SC_RESTORE, SC_MOVE, SC_NEXTWINDOW, SC_PREVWINDOW);
label 1,2;

   function Found(ID : integer) : boolean;
   var I : Integer;
   begin
   result := False;
   for I := Low(IDSARR) to High(IDSARR) do
       if ID = IDSARR[I] then
             begin
             result := True;
             Exit;
             end;
   end;

begin
if not Assigned(Owner) then exit;
//Remove Added Menu Items
SYSMenu := GetsystemMenu((Owner as TCustomForm).Handle, False);
PP := 0;
1:
if Found(GetMenuItemID(SYSMenu,PP)) then begin Inc(PP); goto 1 end
                                    else
                                    begin
                                       if GetMenuItemID(SYSMenu,PP + 1) = SC_CLOSE then begin inc(PP,2); goto 1; end; //don't remove separator before the Close menu item
                                    if not RemoveMenu(SysMenu, PP, MF_BYPOSITION) then exit;
                                    goto 1;
                                    end;
end;

function FindActiveSystemMenu(AOwner : TCustomForm) : TSystemMenu;
var I : integer;
begin
result := nil; //asume failure.
if not Assigned(AOwner) then exit;
for I := 0 to AOwner.ComponentCount - 1 do
    begin
    if AOwner.Components[I].InheritsFrom(TSystemMenu) then
                       begin
                       result := (AOwner.Components[I] as TSystemMenu);
                       Exit;
                       end;
    end;
end;

constructor TSystemMenu.Create(AOwner : TComponent);
resourcestring
SSYSMENUERROR = 'TSystemMenu Error:';
SONLYFORM = 'Can only be created with parent set to a Form.';
SONLYONE = 'Only one instance allowed per TForm object.'+#13#10+'Use the already existent SystemMenu component!';
begin
if AOwner = nil then
  begin
    raise EComponentError.Create(SSYSMENUERROR + #13#10 + SONLYFORM);
  end;
if not AOwner.InheritsFrom(TCustomForm) then
  begin
    raise EComponentError.Create(SSYSMENUERROR + #13#10 + SONLYFORM);
  end;
if FindActiveSystemMenu(AOwner as TCustomForm) <> nil then
  begin
    raise EComponentError.Create(SSYSMENUERROR + #13#10 + SONLYONE);
  end;
inherited Create(AOwner);
DesignPreview := False;
//if not (csDesigning in ComponentState) then
  begin
  smOldFormWndProc := (Owner as TCustomForm).WindowProc;
  (Owner as TCustomForm).WindowProc := smNewFormWindowProc;
  end;
FActive := True;
end;

destructor TSystemMenu.Destroy;
begin
//  if not (csDesigning in ComponentState) then
  begin
    if Assigned(Owner) then
    begin
      (Owner as TCustomForm).WindowProc := smOldFormWndProc;
    end;
  end;
  RemoveItems;
inherited Destroy;
end;

procedure TSystemMenu.smNewFormWindowProc(var Message: TMessage);
var
  I, Item: Integer;
  MenuItem: TMenuItem;
  FindKind: TFindItemKind;
  ContextID: Integer;
  Canvas: TCanvas;
  SaveIndex: Integer;
  DC: HDC;                    
begin
  case Message.Msg of
    WM_SYSCOMMAND:
    begin
      if (Items.Count > 0) and (FActive) then
      begin
        CallClickhandler(Items, Message.WParam);
      end;
    end;
    WM_MENUSELECT:
      with TWMMenuSelect(Message) do
      begin
        FindKind := fkCommand;
        if MenuFlag and MF_POPUP <> 0 then FindKind := fkHandle;
        for I := 0 to Items.Count - 1 do
        begin
          if FindKind = fkHandle then
          begin
            if Menu <> 0 then
              Item := GetSubMenu(Menu, IDItem) else
              Item := -1;
          end
          else
            Item := IDItem;
          MenuItem := FindItem(Item, FindKind);
          if MenuItem <> nil then
          begin
            Application.Hint := GetLongHint(MenuItem.Hint);
            Exit;
          end;
        end;
        Application.Hint := '';
      end;
    WM_INITMENU:
    begin
       if not (FActive) then begin RemoveItems; exit; end;
       if Assigned(OnPopup) then OnPopup(Self);
       if Assigned(OnSysPopup) then OnSysPopup(Self);
       if (csDesigning in ComponentState) then
          if DesignPreview then Refresh
                           else Exit
                                          else
       Refresh;
    end;
    WM_DESTROY:
    begin
      RemoveItems;
    end;
    WM_HELP:
      with PHelpInfo(Message.LParam)^ do
      begin
        for I := 0 to Items.Count - 1 do
        begin
          if hItemHandle = Handle then
            MenuItem := Items
          else
            MenuItem := FindItem(hItemHandle, fkHandle);
          if MenuItem <> nil then
          begin
            ContextID := GetHelpContext(iCtrlID, True);
            if ContextID = 0 then
              ContextID := GetHelpContext(hItemHandle, False);
            if Screen.ActiveForm = nil then Exit;
            if (ContextID = 0) then
              ContextID := Screen.ActiveForm.HelpContext;
            if (biHelp in Screen.ActiveForm.BorderIcons) then
              Application.HelpCommand(HELP_CONTEXTPOPUP, ContextID)
            else
              Application.HelpContext(ContextID);
            Exit;
          end;
        end;
      end;
    WM_DRAWITEM:
      with PDrawItemStruct(Message.LParam)^ do
      begin
        for I := 0 to Items.Count - 1 do
        begin
          MenuItem := FindItem(itemID, fkCommand);
          if MenuItem <> nil then
          begin
            Canvas := TControlCanvas.Create;
            with Canvas do
            try
              SaveIndex := SaveDC(hDC);
              try
                Handle := hDC;
                Font := Screen.MenuFont;
                DrawMenuItem(MenuItem, Canvas, rcItem, TOwnerDrawState(LongRec(itemState).Lo));
              finally
                Handle := 0;
                RestoreDC(hDC, SaveIndex);
              end;
            finally
              Canvas.Free;
            end;
            Exit;
          end;
        end;
      end;
    WM_MEASUREITEM:
      with PMeasureItemStruct(Message.LParam)^ do
      begin
        for I := 0 to Items.Count - 1 do
        begin
          MenuItem := FindItem(itemID, fkCommand);
          if MenuItem <> nil then
          begin
            DC := GetWindowDC((Owner as TCustomForm).Handle);
            try
              Canvas := TControlCanvas.Create;
              with Canvas do
              try
                SaveIndex := SaveDC(DC);
                try
                  Handle := DC;
                  Font := Screen.MenuFont;
                  MeasureItem(MenuItem, Canvas, Integer(itemWidth),
                    Integer(itemHeight));
                finally
                  Handle := 0;
                  RestoreDC(DC, SaveIndex);
                end;
              finally
                Canvas.Free;
              end;
            finally
              ReleaseDC((Owner as TCustomForm).Handle, DC);
            end;
            Exit;
          end;
        end;
      end;
    WM_MENUCHAR:
      for I := 0 to Items.Count - 1 do
        with Self do
          if (Handle = HMENU(Message.LParam)) or
            (FindItem(Message.LParam, fkHandle) <> nil) then
          begin
            ProcessMenuChar(TWMMenuChar(Message));
            Exit;
          end;
  end;
  smOldFormWndProc(Message);
end;

procedure TSystemMenu.CallClickHandler(AMenuItem: TMenuItem; aCommand: Integer);
var I : Integer;
begin
for I := 0 to AMenuItem.Count - 1 do
    begin
    if AMenuItem.Items[I].Command = aCommand then
       begin
       if Assigned(AMenuItem.Items[I].OnClick) then AMenuItem.Items[I].OnClick(AMenuItem.Items[I]);
       Refresh;
       Break;
       end;
    if AMenuItem.Items[I].count <> 0 then
    begin
      CallClickHandler(AMenuItem.Items[I], aCommand);
      Refresh;
    end;
    end;
end;

end.
