unit ToolsManager;

{
  EasyMenus
  Copyright 2005 Murariu Alexandru
  All rights reserved.
  For conditions of distribution and use, see LICENSE.TXT.

}

interface

uses
  Windows, Messages, SysUtils, Classes, Menus, MenuStorage,
  Controls, SystemImageList;

type

  TToolItemClick = procedure (Sender : TObject; Index : integer; Handled : Boolean) of object;

  TToolsManager = class(TComponent)
  private
    FTitle,
    FProgram,
    FWorkingDir,
    FParameters : TStringList;
    ICLICK : TToolItemClick;
    FToolMenuItem : TMenuItem;
    FMenuStorage : TMenuStorage;
    FAutoLoad : boolean;
    FImageList: TSystemImageList;
    FShowIcons : boolean;
    FOnRefreshed: TNotifyEvent;
    function GetCount : integer;
    procedure SetFToolMenuItem(Value : TMenuItem);
    procedure OnItemClick(Sender : TObject);
    function Get(Index: Integer): string;
    procedure Put(Index: Integer; Item: string);
    function Get2(Index: Integer): string;
    procedure Put2(Index: Integer; Item: string);
    function Get3(Index: Integer): string;
    procedure Put3(Index: Integer; Item: string);
    function Get4(Index: Integer): string;
    procedure Put4(Index: Integer; Item: string);
    procedure SETFMenuStorage(Value : TMenuStorage);
    procedure SetFShowIcons(Value : Boolean);
    procedure RemoveItems;
    { Private declarations }
  protected
    procedure Notification(AComponent: TComponent;
    Operation: TOperation); override;
    procedure Loaded; override;
    { Protected declarations }
  public
    constructor Create(AOwner : TComponent); override;
    destructor Destroy; override;
    procedure Add(Title, ProgramPath, WorkingDir, Parameters : string);
    procedure Clear;
    procedure Delete(Index : integer);
    procedure Refresh;
    property Count : integer read GetCount;
    property Title[Index : integer]: string read Get write Put;
    property ProgramFile[Index : integer]: string read Get2 write Put2;
    property WorkingDir[Index : integer]: string read Get3 write Put3;
    property Parameters[Index : integer]: string read Get4 write Put4;
    procedure Save;
    procedure Load;
    { Public declarations }
  published
    property AutoLoad : boolean read FAutoLoad write FAutoLoad;
    property MenuStorage : TMenuStorage read FMenuStorage write SETFMenuStorage;
    property ToolMenuItem : TMenuItem read FToolMenuItem write SetFToolMenuItem;
    property OnToolItemClick : TToolItemClick read ICLICK write ICLICK;
    property ShowIcons : boolean read FShowIcons write SETFShowIcons;
    property OnRefreshed : TNotifyEvent read FOnRefreshed write FOnRefreshed;
    { Published declarations }
  end;

implementation
uses ShellApi;

procedure TToolsManager.SetFShowIcons(Value : Boolean);
begin
if FShowIcons = Value then Exit;
FShowIcons := Value;
Refresh;
end;

procedure TToolsManager.Loaded;
begin
inherited Loaded;
if AutoLoad then Load;
end;

procedure TToolsManager.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  case Operation of
    opInsert:
      begin
      if (AComponent is TMenuStorage) and (MenuStorage = nil) then MenuStorage := (AComponent as TMenuStorage);
      end;
    opRemove:
      begin
      if ToolMenuItem = AComponent then ToolMenuItem := nil;
      if MenuStorage = AComponent then MenuStorage := nil;
      end;
  end;
end;

procedure TToolsManager.Save;
var I : Integer;
begin
if not Assigned(FMenuStorage) then Exit;
FMenuStorage.WriteInteger('Tool_Count',Count);
for i := 0 to Count - 1 do
    begin
    FMenuStorage.WriteString('Title'+IntToStr(I),Title[i]);
    FMenuStorage.WriteString('ProgramFile'+IntToStr(I),ProgramFile[i]);
    FMenuStorage.WriteString('WorkingDir'+IntToStr(I),WorkingDir[i]);
    FMenuStorage.WriteString('Parameters'+IntToStr(I),Parameters[i]);
    end;
end;

procedure TToolsManager.Load;
var I : Integer;
    A,B,C,D : string;
begin
if not Assigned(FMenuStorage) then Exit;
Clear;
for i := 0 to FMenuStorage.ReadInteger('Tool_Count',0,0,0) - 1 do
    begin
    A := FMenuStorage.ReadString('Title'+IntToStr(I),'');
    B := FMenuStorage.ReadString('ProgramFile'+IntToStr(I),'');
    C := FMenuStorage.ReadString('WorkingDir'+IntToStr(I),'');
    D := FMenuStorage.ReadString('Parameters'+IntToStr(I),'');
    Add(A,B,C,D);
    end;
Refresh;
end;

procedure TToolsManager.OnItemClick(Sender : TObject);
var FIndex : integer;
    FHandled : boolean;
begin
FHandled := False;
FIndex := (Sender as TMenuItem).Tag - 1;
if Assigned(OnToolItemClick) then OnToolItemClick(Self, FIndex, FHandled);
if not FHandled then ShellExecute(0, 'open', PChar(FProgram.Strings[FIndex]), PChar(FParameters.Strings[FIndex]), PChar(FWorkingDir.Strings[FIndex]), SW_SHOWNORMAL);
end;

constructor TToolsManager.Create(AOwner : TComponent);
begin
inherited Create(AOwner);
FTitle := TStringList.Create;
FProgram := TStringList.Create;
FWorkingDir := TStringList.Create;
FParameters := TStringList.Create;
FAutoLoad := True;
FShowIcons := True;
FImageList := TSystemImageList.Create(Self);
end;

destructor TToolsManager.Destroy;
begin
RemoveItems;
if Assigned(FImageList) then FImageList.Free;
if Assigned(FParameters) then FParameters.Free;
if Assigned(FWorkingDir) then FWorkingDir.Free;
if Assigned(FProgram) then FProgram.Free;
if Assigned(FTitle) then FTitle.Free;
inherited Destroy;
end;

procedure TToolsManager.SETFMenuStorage(Value : TMenuStorage);
begin
  if FMenuStorage <> Value then
  begin
    FMenuStorage := Value;
    if Value <> nil then Value.FreeNotification(Self);
  end;
end;

procedure TToolsManager.SetFToolMenuItem(Value : TMenuItem);
begin
  if FToolMenuItem <> Value then
  begin
    RemoveItems;
    FToolMenuItem := Value;
    if Value <> nil then Value.FreeNotification(Self);    
    Refresh;
  end;
end;

function TToolsManager.Get(Index: Integer): string;
begin
result := FTitle.Strings[index];
end;

procedure TToolsManager.Put(Index: Integer; Item: string);
begin
FTitle.Strings[index] := Item;
end;

function TToolsManager.Get2(Index: Integer): string;
begin
result := FProgram.Strings[index];
end;

procedure TToolsManager.Put2(Index: Integer; Item: string);
begin
FProgram.Strings[index] := Item;
end;

function TToolsManager.Get3(Index: Integer): string;
begin
result := FWorkingDir.Strings[index];
end;

procedure TToolsManager.Put3(Index: Integer; Item: string);
begin
FWorkingDir.Strings[index] := Item;
end;

function TToolsManager.Get4(Index: Integer): string;
begin
result := FParameters.Strings[index];
end;

procedure TToolsManager.Put4(Index: Integer; Item: string);
begin
FParameters.Strings[index] := Item;
end;

function TToolsManager.GetCount : integer;
begin
result := FTitle.Count;
end;

procedure TToolsManager.Add(Title, ProgramPath, WorkingDir, Parameters : string);
begin
if (Title = '') or (ProgramPath = '') then exit;
FTitle.Add(Title);
FProgram.Add(ProgramPath);
FWorkingDir.Add(WorkingDir);
FParameters.Add(Parameters);
Refresh;
end;

procedure TToolsManager.Clear;
begin
FTitle.Clear;
FProgram.Clear;
FWorkingDir.Clear;
FParameters.Clear;
Refresh;
end;

procedure TToolsManager.Delete(Index : integer);
begin
FTitle.Delete(Index);
FProgram.Delete(Index);
FWorkingDir.Delete(Index);
FParameters.Delete(Index);
Refresh;
end;

procedure TToolsManager.RemoveItems;
var I : integer;
label 1;
begin
if not Assigned(FToolMenuItem) then exit;
1:for I := 0 to FToolMenuItem.Count - 1 do
    if FToolMenuItem.Items[I].Tag <> 0 then
               begin
               FToolMenuItem.Items[i].Free;
               goto 1;
               end;
end;

procedure TToolsManager.Refresh;
var I,C : integer;
    MI : TMenuItem;
begin
if csDesigning in Self.ComponentState then exit;
RemoveItems;
if not Assigned(FToolMenuItem) then exit;
if FTitle.Count = 0 then exit;
C := FTitle.Count;
if ShowIcons then FToolMenuItem.SubMenuImages := FImageList
             else FToolMenuItem.SubMenuImages := nil;
for I := 0 to C - 1 do
    begin
    MI := TMenuItem.Create(nil);
    MI.Caption := FTitle.Strings[I];
    if ShowIcons then
                 begin
                 if FileExists(FProgram.Strings[i]) or DirectoryExists(FProgram.Strings[i])
                                                    then MI.ImageIndex := FImageList.GetImageIndex(FProgram.Strings[i])
                                                    else MI.ImageIndex := -1;
                 end;
    MI.Tag := I + 1;
    MI.OnClick := OnItemClick;
    FToolMenuItem.Insert(I,MI);
    end;
if Assigned(OnRefreshed) then OnRefreshed(Self);    
end;

end.
