unit ToolsManagerEditor;

{
  EasyMenus
  Copyright 2005 Murariu Alexandru
  muralexandru@yahoo.com
  All rights reserved.
  For conditions of distribution and use, see LICENSE.TXT.

}

interface

uses
  Windows, Messages, SysUtils, Classes, ComCtrls,
  Controls, ToolsManager, Forms, Dialogs;

type
  TToolsManagerEditor = class(TComponent)
  private
  FToolsManager : TToolsManager;
  FHelpType : THelpType;
  FHelpKeyword : String;
  FHelpContext : THelpContext;
  FMacros : TStrings;
    FHelpKeyword_2: string;
    FHelpContext_2: THelpContext;
    FHelpType_2: THelpType;
  procedure SetHelpContext(const Value: THelpContext);
  procedure SetHelpKeyword(const Value: String);
  procedure SETFMacros(Value : TStrings);
    procedure SetHelpContext_2(const Value: THelpContext);
    procedure SetHelpKeyword_2(const Value: string);
    { Private declarations }
  protected
  procedure Notification(AComponent: TComponent;
  Operation: TOperation); override;
    { Protected declarations }
  public
  function Edit : boolean;
  constructor Create(AOwner : TComponent); override;
  destructor Destroy; override;
    { Public declarations }
  published
  property Macros : TStrings read FMacros write SETFMacros;
  property HelpType : THelpType read FHelpType write FHelpType;// default htContext;
  property HelpKeyword : string read FHelpKeyword write SetHelpKeyword stored True;
  property HelpContext : THelpContext read FHelpContext write SetHelpContext stored True default 0;
  property HelpType_2 : THelpType read FHelpType_2 write FHelpType_2;// default htContext;
  property HelpKeyword_2 : string read FHelpKeyword_2 write SetHelpKeyword_2 stored True;
  property HelpContext_2 : THelpContext read FHelpContext_2 write SetHelpContext_2 stored True default 0;
  property ToolsManager : TToolsManager read FToolsManager write FToolsManager;
    { Published declarations }
  end;

implementation
uses ToolsManagerEditor_Form_unit;

{$R ToolsManagerEditor_R.res}

procedure TToolsManagerEditor.SETFMacros(Value : TStrings);
begin
FMacros.Assign(Value);
end;

constructor TToolsManagerEditor.Create(AOwner : TComponent);
begin
inherited Create(AOwner);
FMacros := TStringlist.Create;
HelpType := htContext;
HelpType_2 := htContext;
end;

destructor TToolsManagerEditor.Destroy;
begin
if Assigned(FMacros) then FMacros.Free;
inherited Destroy;
end;

procedure TToolsManagerEditor.SetHelpContext(const Value: THelpContext);
begin
  if not (csLoading in ComponentState) then FHelpType := htContext;
  FHelpContext := Value;
end;

procedure TToolsManagerEditor.SetHelpKeyword(const Value: String);
begin
  if not (csLoading in ComponentState) then FHelpType := htKeyword;
  FHelpKeyword := Value;
end;

procedure TToolsManagerEditor.SetHelpContext_2(const Value: THelpContext);
begin
  if not (csLoading in ComponentState) then FHelpType_2 := htContext;
  FHelpContext_2 := Value;
end;

procedure TToolsManagerEditor.SetHelpKeyword_2(const Value: string);
begin
  if not (csLoading in ComponentState) then FHelpType_2 := htKeyword;
  FHelpKeyword_2 := Value;
end;

procedure TToolsManagerEditor.Notification(AComponent: TComponent;
  Operation: TOperation);
begin
  inherited Notification(AComponent, Operation);
  case Operation of
    opInsert:
      begin
      if (AComponent is TToolsManager) and (ToolsManager = nil) then ToolsManager := (AComponent as TToolsManager);
      end;
    opRemove:
      begin
      if ToolsManager = AComponent then ToolsManager := nil;
      end;
  end;
end;

function TToolsManagerEditor.Edit : boolean;
var I : integer;
    L : TListItem;
    ToolsManagerEditor_Form : TToolsManagerEditor_Form;
begin
result := False;
if not Assigned(FToolsManager) then exit;
ToolsManagerEditor_Form := TToolsManagerEditor_Form.Create(nil);
ToolsManagerEditor_Form.HelpKeyword := HelpKeyword;
ToolsManagerEditor_Form.HelpContext := HelpContext;
ToolsManagerEditor_Form.HelpType := HelpType;
ToolsManagerEditor_Form.HelpKeyword_2 := HelpKeyword_2;
ToolsManagerEditor_Form.HelpContext_2 := HelpContext_2;
ToolsManagerEditor_Form.HelpType_2 := HelpType_2;
ToolsManagerEditor_Form.Macros.Assign(Macros);
ToolsManagerEditor_Form.ListView1.Clear;
ToolsManagerEditor_Form.ShowImages := FToolsManager.ShowIcons;
for I := 0 to FToolsManager.Count - 1 do
    begin
    with ToolsManagerEditor_Form do
        begin
        L := ListView1.Items.Add;
        L.Caption := FToolsManager.Title[i];
        L.SubItems.Add(FToolsManager.ProgramFile[i]);
        L.SubItems.Add(FToolsManager.WorkingDir[i]);
        L.SubItems.Add(FToolsManager.Parameters[i]);
        end;
    end;
ToolsManagerEditor_Form.Position := poScreenCenter;
if ToolsManagerEditor_Form.ShowModal <> mrok then
                                             begin
                                             ToolsManagerEditor_Form.Free;
                                             Exit;
                                             end;
FToolsManager.Clear;
for i := 0 to ToolsManagerEditor_Form.ListView1.Items.Count - 1 do
    begin
    L := ToolsManagerEditor_Form.ListView1.Items.Item[i];
    FToolsManager.Add(L.Caption, L.SubItems.Strings[0], L.SubItems.Strings[1], L.SubItems.Strings[2]);
    end;
FToolsManager.Refresh;
Result := True;
ToolsManagerEditor_Form.Free;
end;

end.
