unit ToolsManagerEditor_Form_unit;

{
  EasyMenus
  Copyright 2005 Murariu Alexandru
  muralexandru@yahoo.com
  All rights reserved.
  For conditions of distribution and use, see LICENSE.TXT.

}

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, ComCtrls, Buttons, ImgList, SystemImageList;

type
  TToolsManagerEditor_Form = class(TForm)
    Label1: TLabel;
    ListView1: TListView;
    Button1: TButton;
    Button2: TButton;
    Button3: TButton;
    Button4: TButton;
    Button5: TButton;
    Button6: TButton;
    SystemImageList1: TSystemImageList;
    UpBtn: TSpeedButton;
    DownBtn: TSpeedButton;
    procedure UpBtnClick(Sender: TObject);
    procedure DownBtnClick(Sender: TObject);
    procedure Button1Click(Sender: TObject);
    procedure Button3Click(Sender: TObject);
    procedure Button2Click(Sender: TObject);
    procedure ListView1Change(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure FormShow(Sender: TObject);
    procedure Button6Click(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure ListView1GetImageIndex(Sender: TObject; Item: TListItem);
  private
    { Private declarations }
  public
  Macros : TStringList;
  ShowImages : Boolean;
  HelpType_2 : THelpType;
  HelpKeyword_2 : string;
  HelpContext_2 : THelpContext;
  procedure CheckEnabled;
  procedure SplitCapDesc(const T : string; var S1,S2 : string);
  procedure AddEditItem(Edit : Boolean);
    { Public declarations }
  end;

implementation

uses ToolManagerEditor_SingleItem_unit;

{$R *.dfm}

procedure TToolsManagerEditor_Form.UpBtnClick(Sender: TObject);
var L,L2  :TListitem;
begin
try
L:=Listview1.selected;
L2:=Listview1.Items.Insert(L.Index-1);
L2.Assign(L);
L2.Selected:=True;
L2.MakeVisible(False);
L.Free;
except
end;
end;

procedure TToolsManagerEditor_Form.DownBtnClick(Sender: TObject);
var L,L2:TListItem;
begin
try
L:=Listview1.selected;
L2:=Listview1.Items.Insert(L.Index+2);
L2.Assign(L);
L2.Selected:=True;
L2.MakeVisible(False);
L.Free;
except
end;
end;

procedure TToolsManagerEditor_Form.AddEditItem(Edit : Boolean);
var L : TListitem;
    ToolManagerEditor_SingleItem : TToolManagerEditor_SingleItem;
    I : Integer;
    S1, S2 : string;
begin
ToolManagerEditor_SingleItem := TToolManagerEditor_SingleItem.Create(nil);
ToolManagerEditor_SingleItem.HelpKeyword := HelpKeyword_2;
ToolManagerEditor_SingleItem.HelpContext := HelpContext_2;
ToolManagerEditor_SingleItem.HelpType := HelpType_2;
with ToolManagerEditor_SingleItem do
     begin
     if not Edit then
       begin
       Edit1.Text := '';
       Edit2.Text := '';
       Edit3.Text := '';
       Edit4.Text := '';
       end
       else
       begin
       L := ListView1.Selected;
       if L = nil then Exit;
       Edit1.Text := L.Caption;
       Edit2.Text := L.SubItems.Strings[0];
       Edit3.Text := L.SubItems.Strings[1];
       Edit4.Text := L.SubItems.Strings[2];
       end;
     Position := poScreenCenter;
     if Macros.Count > 0 then
     begin
     MacroList.Items.Clear;
     for I := 0 to Macros.Count - 1 do
         begin
         with MacroList.Items.Add do
              begin
              SplitCapDesc(Macros.Strings[i], S1, S2);
              Caption := S1;
              SubItems.Add(S2);
              end;
         end;
     MacroList.Items.Item[0].Selected := True;
     end;
     if ShowModal <> mrok then
                          begin
                          ToolManagerEditor_SingleItem.Free;
                          Exit;
                          end;
     if not Edit then
       begin
       L := ListView1.Items.Add;
       L.Caption := Edit1.Text;
       L.SubItems.Add(Edit2.Text);
       L.SubItems.Add(Edit3.Text);
       L.SubItems.Add(Edit4.Text);
       L.Selected := True;
       L.MakeVisible(False);
       end
       else
       begin
       L.Caption := Edit1.Text;
       L.SubItems.Strings[0] := Edit2.Text;
       L.SubItems.Strings[1] := Edit3.Text;
       L.SubItems.Strings[2] := Edit4.Text;
       end;
     end;
ToolManagerEditor_SingleItem.Free;
end;

procedure TToolsManagerEditor_Form.Button1Click(Sender: TObject);
begin
AddEditItem(False);
end;

procedure TToolsManagerEditor_Form.Button3Click(Sender: TObject);
var L : TListItem;
    Index : integer;
begin
L := ListView1.Selected;
if L = nil then Exit;
Index := L.Index - 1;
ListView1.DeleteSelected;
if ListView1.Items.Count = 0 then Exit;
if Index < 0 then Index := 0;
ListView1.Items.Item[Index].Selected := True;
end;

procedure TToolsManagerEditor_Form.SplitCapDesc(const T : string; var S1,S2 : string);
begin
S1 := T;
S2 := '';
if Pos('|',T) = 0 then Exit;
S1 := Copy(T,1,Pos('|',T) - 1);
S2 := Copy(T,Pos('|',T) + 1,Length(T) - Pos('|',T) + 1);
end;

procedure TToolsManagerEditor_Form.Button2Click(Sender: TObject);
begin
AddEditItem(True);
end;

procedure TToolsManagerEditor_Form.CheckEnabled;
var E : boolean;
begin
E := (ListView1.Selected <> nil);
Button2.Enabled := E;
Button3.Enabled := E;
UpBtn.Enabled := E;
DownBtn.Enabled := E;
end;

procedure TToolsManagerEditor_Form.ListView1Change(Sender: TObject;
  Item: TListItem; Change: TItemChange);
begin
CheckEnabled;
end;

procedure TToolsManagerEditor_Form.FormShow(Sender: TObject);
begin
CheckEnabled;
Button6.Enabled := ((HelpType = htKeyword) and (HelpKeyword <> '')) or ((HelpType = htContext) and (HelpContext <> 0));
end;

procedure TToolsManagerEditor_Form.Button6Click(Sender: TObject);
begin
if ((HelpType = htKeyword) and (HelpKeyword <> '')) then Application.HelpKeyword(HelpKeyword);
if ((HelpType = htContext) and (HelpContext <> 0))  then Application.HelpContext(HelpContext);
end;

procedure TToolsManagerEditor_Form.FormCreate(Sender: TObject);
begin
Macros := TStringList.Create;
try
UpBtn.Glyph.LoadFromResourceName(HInstance, 'MACRO_UP');
DownBtn.Glyph.LoadFromResourceName(HInstance, 'MACRO_DOWN');
except
end;
end;

procedure TToolsManagerEditor_Form.FormDestroy(Sender: TObject);
begin
Macros.Free;
end;

procedure TToolsManagerEditor_Form.ListView1GetImageIndex(Sender: TObject;
  Item: TListItem);
begin
if ShowImages then if FileExists(Item.SubItems.Strings[0]) or DirectoryExists(Item.SubItems.Strings[0]) then Item.ImageIndex := SystemImageList1.GetImageIndex(Item.SubItems.Strings[0])
       else Item.ImageIndex := -1 else Item.ImageIndex := -1;
end;

end.
